"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTemplate = void 0;
const getPromptsQuestions_1 = require("./getPromptsQuestions");
const path_1 = __importDefault(require("path"));
const copyFolderSync_1 = require("../utils/copyFolderSync");
const child_process_1 = require("child_process");
const log_1 = require("../utils/log");
const writeFileSync_1 = require("../utils/writeFileSync");
const app_1 = require("../app");
const colby_1 = require("colby");
const getVitePackageConfig_1 = require("./vite/getVitePackageConfig");
const getViteDefaultConfig_1 = require("./vite/getViteDefaultConfig");
const getViteIndexHTML_1 = require("./vite/getViteIndexHTML");
const getWebPackIndexHTML_1 = require("./webpack/getWebPackIndexHTML");
const getWebpackPackageConfig_1 = require("./webpack/getWebpackPackageConfig");
const getWebpackDefaultConfig_1 = require("./webpack/getWebpackDefaultConfig");
const getMainFileCode_1 = require("./getMainFileCode");
const getWebpackTsConfig_1 = require("./webpack/getWebpackTsConfig");
const getViteTsConfig_1 = require("./vite/getViteTsConfig");
const package_json_1 = __importDefault(require("../package.json"));
const createTemplate = (argvAppName) => __awaiter(void 0, void 0, void 0, function* () {
    const { version: kixVersion, dependencies: { typescript: typescriptVersion } } = package_json_1.default;
    const { appName, templateType, moduleBuilder } = yield (0, getPromptsQuestions_1.getPromptsQuestions)(argvAppName);
    const toPath = path_1.default.resolve(app_1.runDirectory, appName);
    const fromPath = path_1.default.resolve(path_1.default.join(app_1.appDirectory, "templates", "config"));
    (0, copyFolderSync_1.copyFolderSync)(fromPath, toPath, [], true);
    const packageJsonPath = path_1.default.resolve(toPath, "package.json");
    const fileExt = (templateType === getPromptsQuestions_1.TemplateTypes.JS ? "js" : "ts");
    const mainFileName = ("./src/index." + fileExt + "x");
    if (moduleBuilder === getPromptsQuestions_1.ModuleBuilder.WEBPACK) {
        (0, writeFileSync_1.writeFileSync)(packageJsonPath, JSON.stringify((0, getWebpackPackageConfig_1.getWebpackPackageConfig)(kixVersion, typescriptVersion, appName), null, 4));
        const webpackConfigPath = path_1.default.resolve(path_1.default.join(toPath, "webpack.config.js"));
        (0, writeFileSync_1.writeFileSync)(webpackConfigPath, (0, getWebpackDefaultConfig_1.getWebpackDefaultConfig)(mainFileName));
        (0, writeFileSync_1.writeFileSync)(path_1.default.resolve(path_1.default.join(toPath, "index.html")), (0, getWebPackIndexHTML_1.getWebPackIndexHTML)());
        (0, writeFileSync_1.writeFileSync)(path_1.default.resolve(path_1.default.join(toPath, "tsconfig.json")), JSON.stringify((0, getWebpackTsConfig_1.getWebpackTsConfig)(), null, 4));
    }
    else if (moduleBuilder === getPromptsQuestions_1.ModuleBuilder.VITE) {
        (0, writeFileSync_1.writeFileSync)(packageJsonPath, JSON.stringify((0, getVitePackageConfig_1.getVitePackageConfig)(kixVersion, typescriptVersion, appName), null, 4));
        const viteConfigPath = path_1.default.resolve(path_1.default.join(toPath, "vite.config." + fileExt));
        (0, writeFileSync_1.writeFileSync)(viteConfigPath, (0, getViteDefaultConfig_1.getViteDefaultConfig)());
        (0, writeFileSync_1.writeFileSync)(path_1.default.resolve(path_1.default.join(toPath, "index.html")), (0, getViteIndexHTML_1.getViteIndexHTML)(mainFileName));
        (0, writeFileSync_1.writeFileSync)(path_1.default.resolve(path_1.default.join(toPath, "tsconfig.json")), JSON.stringify((0, getViteTsConfig_1.getVitTsConfig)(), null, 4));
    }
    const executeMainFilePath = path_1.default.resolve(path_1.default.join(toPath, mainFileName));
    (0, writeFileSync_1.writeFileSync)(executeMainFilePath, (0, getMainFileCode_1.getMainFileCode)());
    (0, child_process_1.spawn)('npm', ["install"], {
        shell: true,
        cwd: toPath,
        stdio: "inherit",
    })
        .on("error", log_1.log.error)
        .on("close", () => {
        console.clear();
        log_1.log.log((0, colby_1.toRgb)(0, 128, 0)(`Project "${appName}" Created`), `\nLocation: ` + toPath);
    });
});
exports.createTemplate = createTemplate;
