"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createObject = void 0;
const typescript_1 = __importDefault(require("typescript"));
const identifier_1 = require("./identifier");
const stringLiteral_1 = require("./stringLiteral");
const createObject = (objectPropertiesNodes) => {
    const factory = typescript_1.default.factory;
    const saferPropertyRegexp = /^[a-zA-Z_]+$/;
    return factory.createObjectLiteralExpression(objectPropertiesNodes.map((node) => {
        if (node instanceof Array) {
            let propertyNameNode = node[0];
            let propertyNode;
            if (typeof propertyNameNode === "string" || typescript_1.default.isIdentifier(propertyNameNode)) {
                propertyNameNode = typeof propertyNameNode === "string" ? propertyNameNode : typescript_1.default.idText(propertyNameNode);
                propertyNode = saferPropertyRegexp.test(propertyNameNode) ? (0, identifier_1.identifier)(propertyNameNode) : (0, stringLiteral_1.stringLiteral)(propertyNameNode);
            }
            else {
                propertyNode = factory.createComputedPropertyName(propertyNameNode);
            }
            return factory.createPropertyAssignment(propertyNode, node[1]);
        }
        else if (typescript_1.default.isObjectLiteralElementLike(node)) {
            return node;
        }
        return factory.createSpreadAssignment(factory.createParenthesizedExpression((0, identifier_1.identifier)(node)));
    }), false);
};
exports.createObject = createObject;
