"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BinaryExpression = void 0;
const typescript_1 = __importDefault(require("typescript"));
const nodeToken_1 = require("../factoryCode/nodeToken");
const propertyAccessExpression_1 = require("../factoryCode/propertyAccessExpression");
const AssignmentTokensList = [
    typescript_1.default.SyntaxKind.EqualsToken,
    typescript_1.default.SyntaxKind.AsteriskEqualsToken,
    typescript_1.default.SyntaxKind.AsteriskAsteriskEqualsToken,
    typescript_1.default.SyntaxKind.SlashEqualsToken,
    typescript_1.default.SyntaxKind.PercentEqualsToken,
    typescript_1.default.SyntaxKind.PlusEqualsToken,
    typescript_1.default.SyntaxKind.MinusEqualsToken,
    typescript_1.default.SyntaxKind.LessThanLessThanEqualsToken,
    typescript_1.default.SyntaxKind.GreaterThanGreaterThanEqualsToken,
    typescript_1.default.SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken,
    typescript_1.default.SyntaxKind.AmpersandEqualsToken,
    typescript_1.default.SyntaxKind.CaretEqualsToken,
    typescript_1.default.SyntaxKind.BarEqualsToken,
    typescript_1.default.SyntaxKind.AmpersandAmpersandEqualsToken,
    typescript_1.default.SyntaxKind.BarBarEqualsToken,
    typescript_1.default.SyntaxKind.QuestionQuestionEqualsToken,
];
const BinaryExpression = (node, visitor, context) => {
    let visitedNode = typescript_1.default.visitEachChild(node, visitor, context);
    if (typescript_1.default.isIdentifier(visitedNode.left) && AssignmentTokensList.includes(visitedNode.operatorToken.kind)) {
        const identifierName = typescript_1.default.idText(visitedNode.left);
        context.addIdentifiersChannelCallback(identifierName, (identifierState) => {
            identifierState.isChanged = true;
            identifierState.substituteCallback = (indexIdToUniqueString, declarationIdentifier) => {
                context.substituteNodesList.set(visitedNode, () => {
                    return (0, nodeToken_1.nodeToken)([
                        (0, propertyAccessExpression_1.propertyAccessExpression)([
                            declarationIdentifier,
                            indexIdToUniqueString
                        ], "createPropertyAccessExpression"),
                        context.factory.createParenthesizedExpression(context.factory.createBinaryExpression(visitedNode.left, visitedNode.operatorToken, visitedNode.right))
                    ]);
                });
            };
        });
    }
    return visitedNode;
};
exports.BinaryExpression = BinaryExpression;
