"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DoStatement = void 0;
const typescript_1 = __importDefault(require("typescript"));
const createObject_1 = require("../factoryCode/createObject");
const variableStatement_1 = require("../factoryCode/variableStatement");
const createBlockVisitor_1 = require("./utils/createBlockVisitor");
const doWhileBlockVisitor = (0, createBlockVisitor_1.createBlockVisitor)((statement, visitor, context) => {
    return visitor(statement);
}, false);
const DoStatement = (node, visitor, context) => {
    const [visitedStatementNode, variableState] = doWhileBlockVisitor(node.statement, visitor, context);
    const expression = visitor(node.expression);
    return context.factory.updateDoStatement(node, updateStatement(visitedStatementNode, variableState, context), expression);
};
exports.DoStatement = DoStatement;
const updateStatement = (statement, { blockScopeIdentifiers }, context) => {
    if (!blockScopeIdentifiers)
        return statement;
    const variableDeclarationNode = (0, variableStatement_1.variableStatement)([
        [
            blockScopeIdentifiers,
            (0, createObject_1.createObject)([])
        ],
    ], typescript_1.default.NodeFlags.Const);
    if (typescript_1.default.isBlock(statement)) {
        return context.factory.updateBlock(statement, [
            variableDeclarationNode,
            ...statement.statements
        ]);
    }
    return context.factory.createBlock([
        variableDeclarationNode,
        statement
    ]);
};
