"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ForOfStatement = void 0;
const typescript_1 = __importDefault(require("typescript"));
const createObject_1 = require("../factoryCode/createObject");
const variableStatement_1 = require("../factoryCode/variableStatement");
const getVariableDeclarationNames_1 = require("../utils/getVariableDeclarationNames");
const createBlockVisitor_1 = require("./utils/createBlockVisitor");
const ForInStatementVisitor = (0, createBlockVisitor_1.createBlockVisitor)(({ initializer, statement }, visitor, context) => {
    const defaultDeclarations = [];
    if (typescript_1.default.isVariableDeclarationList(initializer)) {
        for (const variableDeclaration of initializer.declarations) {
            const declarationNamesObject = (0, getVariableDeclarationNames_1.getVariableDeclarationNames)(variableDeclaration);
            for (const declarationIdentifierName in declarationNamesObject) {
                context.addDeclaredIdentifierState(declarationIdentifierName);
                context.addIdentifiersChannelCallback(declarationIdentifierName, (identifierState) => {
                    identifierState.declaredFlag = initializer.flags;
                    identifierState.substituteCallback = (indexIdToUniqueString, declarationIdentifier) => {
                        if (initializer.flags !== typescript_1.default.NodeFlags.None) {
                            defaultDeclarations.push([
                                indexIdToUniqueString,
                                context.factory.createIdentifier(declarationIdentifierName)
                            ]);
                        }
                    };
                });
            }
        }
    }
    return {
        defaultDeclarations: defaultDeclarations,
        statement: visitor(statement)
    };
}, false);
const ForOfStatement = (node, visitor, context) => {
    const [{ statement, defaultDeclarations }, variableState] = ForInStatementVisitor(node, visitor, context);
    const updatedStatement = updateForOfStatementStatement(statement, variableState, defaultDeclarations, context);
    return context.factory.updateForOfStatement(node, node.awaitModifier && visitor(node.awaitModifier), visitor(node.initializer), visitor(node.expression), updatedStatement);
};
exports.ForOfStatement = ForOfStatement;
const updateForOfStatementStatement = (statement, { blockScopeIdentifiers }, defaultDeclarations, context) => {
    if (!blockScopeIdentifiers)
        return statement;
    const variableDeclarationNode = (0, variableStatement_1.variableStatement)([
        [
            blockScopeIdentifiers,
            (0, createObject_1.createObject)(defaultDeclarations)
        ],
    ], typescript_1.default.NodeFlags.Const);
    if (typescript_1.default.isBlock(statement)) {
        return context.factory.updateBlock(statement, [
            variableDeclarationNode,
            ...statement.statements
        ]);
    }
    return context.factory.createBlock([
        variableDeclarationNode,
        statement
    ]);
};
