"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ForStatement = void 0;
const typescript_1 = __importDefault(require("typescript"));
const createObject_1 = require("../factoryCode/createObject");
const nodeToken_1 = require("../factoryCode/nodeToken");
const variableStatement_1 = require("../factoryCode/variableStatement");
const getVariableDeclarationNames_1 = require("../utils/getVariableDeclarationNames");
const createBlockVisitor_1 = require("./utils/createBlockVisitor");
const ForStatementBlockVisitor = (0, createBlockVisitor_1.createBlockVisitor)(({ node: { statement, initializer }, getVariableUniqueIdentifierParent }, visitor, context) => {
    const getVariableUniqueIdentifierCache = context.getVariableUniqueIdentifier;
    context.getVariableUniqueIdentifier = getVariableUniqueIdentifierParent;
    statement = visitor(statement);
    context.getVariableUniqueIdentifier = getVariableUniqueIdentifierCache;
    return statement;
}, false);
const ForStatementVisitor = (0, createBlockVisitor_1.createBlockVisitor)((node, visitor, context) => {
    let { initializer, condition, incrementor } = node;
    initializer = initializer && visitor(initializer);
    condition = condition && visitor(condition);
    incrementor = incrementor && visitor(incrementor);
    if (initializer && typescript_1.default.isVariableDeclarationList(initializer)) {
        for (const variableDeclaration of initializer.declarations) {
            const declarationNamesObject = (0, getVariableDeclarationNames_1.getVariableDeclarationNames)(variableDeclaration);
            for (const declarationIdentifierName in declarationNamesObject) {
                context.addDeclaredIdentifierState(declarationIdentifierName);
                context.addIdentifiersChannelCallback(declarationIdentifierName, (identifierState) => {
                    identifierState.declaredFlag = initializer.flags;
                });
            }
        }
    }
    const [statement, _] = ForStatementBlockVisitor({
        node,
        getVariableUniqueIdentifierParent: context.getVariableUniqueIdentifier
    }, visitor, context);
    return {
        initializer,
        condition,
        incrementor,
        statement
    };
}, false);
const ForStatement = (node, visitor, context) => {
    let [{ initializer, condition, incrementor, statement }, variableState] = ForStatementVisitor(node, visitor, context);
    if (initializer && typescript_1.default.isVariableDeclarationList(initializer) && initializer.flags === typescript_1.default.NodeFlags.Let) {
        const letInitializerUpdated = updateLetInitializerAndConditionForBlock(initializer, condition, variableState, context);
        initializer = letInitializerUpdated.initializer;
        condition = letInitializerUpdated.condition;
    }
    else {
        statement = updateStatementNode(statement, variableState, context);
    }
    return context.factory.updateForStatement(node, initializer, condition, incrementor, statement);
};
exports.ForStatement = ForStatement;
const updateLetInitializerAndConditionForBlock = (initializer, condition, { blockScopeIdentifiers }, context) => {
    if (!blockScopeIdentifiers)
        return { initializer, condition };
    const declarationNode = context.factory.createVariableDeclaration(blockScopeIdentifiers, undefined, undefined, undefined);
    initializer = context.factory.updateVariableDeclarationList(initializer, [
        declarationNode,
        ...initializer.declarations
    ]);
    const stateObjectNode = (0, nodeToken_1.nodeToken)([
        blockScopeIdentifiers,
        (0, createObject_1.createObject)([])
    ], typescript_1.default.SyntaxKind.EqualsToken);
    if (condition) {
        condition = (0, nodeToken_1.nodeToken)([stateObjectNode, condition], typescript_1.default.SyntaxKind.CommaToken);
    }
    else {
        condition = stateObjectNode;
    }
    return { initializer, condition };
};
const updateStatementNode = (statement, { blockScopeIdentifiers }, context) => {
    if (!blockScopeIdentifiers)
        return statement;
    const variableDeclarationNode = (0, variableStatement_1.variableStatement)([
        [
            blockScopeIdentifiers,
            (0, createObject_1.createObject)([])
        ],
    ], typescript_1.default.NodeFlags.Const);
    if (typescript_1.default.isBlock(statement)) {
        return context.factory.updateBlock(statement, [
            variableDeclarationNode,
            ...statement.statements
        ]);
    }
    return context.factory.createBlock([
        variableDeclarationNode,
        statement
    ]);
};
