"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FunctionDeclaration = void 0;
const typescript_1 = __importDefault(require("typescript"));
const identifier_1 = require("../factoryCode/identifier");
const nodeToken_1 = require("../factoryCode/nodeToken");
const propertyAccessExpression_1 = require("../factoryCode/propertyAccessExpression");
const createGlobalBlockNodesVisitor_1 = require("./utils/createGlobalBlockNodesVisitor");
const FunctionDeclarationVisitor = (0, createGlobalBlockNodesVisitor_1.createGlobalBlockNodesVisitor)((visitedNode, declarationNode, context) => {
    return context.factory.updateFunctionDeclaration(visitedNode, typescript_1.default.getModifiers(visitedNode), visitedNode.asteriskToken, visitedNode.name, visitedNode.typeParameters, visitedNode.parameters, visitedNode.type, visitedNode.body && context.factory.updateBlock(visitedNode.body, [
        declarationNode,
        ...visitedNode.body.statements
    ]));
});
const FunctionDeclaration = (node, visitor, context) => {
    const returnValue = [
        FunctionDeclarationVisitor(node, visitor, context)
    ];
    if (node.name) {
        const declarationIdentifierName = typescript_1.default.idText(node.name);
        context.addIdentifiersChannelCallback(declarationIdentifierName, (identifierState) => {
            identifierState.declaredFlag = typescript_1.default.NodeFlags.None;
            const declarationMarker = context.factory.createIdentifier("");
            returnValue.push(declarationMarker);
            identifierState.substituteCallback = (indexIdToUniqueString, declarationIdentifier) => {
                context.substituteNodesList.set(declarationMarker, () => {
                    return context.factory.createExpressionStatement((0, nodeToken_1.nodeToken)([
                        (0, propertyAccessExpression_1.propertyAccessExpression)([
                            declarationIdentifier,
                            indexIdToUniqueString
                        ], "createPropertyAccessExpression"),
                        (0, identifier_1.identifier)(declarationIdentifierName)
                    ]));
                });
            };
        });
    }
    return returnValue;
};
exports.FunctionDeclaration = FunctionDeclaration;
