"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SwitchStatement = void 0;
const typescript_1 = __importDefault(require("typescript"));
const createObject_1 = require("../factoryCode/createObject");
const variableStatement_1 = require("../factoryCode/variableStatement");
const createBlockVisitor_1 = require("./utils/createBlockVisitor");
const SwitchStatementBlockVisitor = (0, createBlockVisitor_1.createBlockVisitor)((nodes, visitor, context) => {
    return typescript_1.default.visitNodes(nodes, visitor);
}, false);
const SwitchStatement = (node, visitor, context) => {
    const clauses = node.caseBlock.clauses.map((caseBlockNode) => {
        const [visitedStatementNode, variableState] = SwitchStatementBlockVisitor(caseBlockNode.statements, visitor, context);
        const updatedStatements = updateCaseOrDefaultClauseStatements(context.factory.createNodeArray(visitedStatementNode), variableState);
        if (typescript_1.default.isDefaultClause(caseBlockNode)) {
            return context.factory.updateDefaultClause(caseBlockNode, updatedStatements);
        }
        return context.factory.updateCaseClause(caseBlockNode, visitor(caseBlockNode.expression), updatedStatements);
    });
    return context.factory.updateSwitchStatement(node, visitor(node.expression), context.factory.updateCaseBlock(node.caseBlock, clauses));
};
exports.SwitchStatement = SwitchStatement;
const updateCaseOrDefaultClauseStatements = (statements, variableState) => {
    if (variableState.blockScopeIdentifiers) {
        return [
            (0, variableStatement_1.variableStatement)([
                [variableState.blockScopeIdentifiers, (0, createObject_1.createObject)([])],
            ]),
            ...statements,
        ];
    }
    return statements;
};
