"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TryStatement = void 0;
const createObject_1 = require("../factoryCode/createObject");
const identifier_1 = require("../factoryCode/identifier");
const variableStatement_1 = require("../factoryCode/variableStatement");
const getVariableDeclarationNames_1 = require("../utils/getVariableDeclarationNames");
const createBlockVisitor_1 = require("./utils/createBlockVisitor");
const TryStatementVisitor = (0, createBlockVisitor_1.createBlockVisitor)((node, visitor, context) => {
    return visitor(node);
}, false);
const TryStatementCatchClauseVisitor = (0, createBlockVisitor_1.createBlockVisitor)((node, visitor, context) => {
    let propertyDeclaration = [];
    if (node.variableDeclaration) {
        const declarationNamesObject = (0, getVariableDeclarationNames_1.getVariableDeclarationNames)(node.variableDeclaration);
        for (const declarationIdentifierName in declarationNamesObject) {
            context.addDeclaredIdentifierState(declarationIdentifierName);
            context.addIdentifiersChannelCallback(declarationIdentifierName, (identifierState) => {
                identifierState.substituteCallback = (indexIdToUniqueString, declarationIdentifier) => {
                    propertyDeclaration.push([
                        indexIdToUniqueString,
                        (0, identifier_1.identifier)(declarationIdentifierName)
                    ]);
                };
            });
        }
    }
    return {
        propertyDeclaration,
        visitedNode: visitor(node)
    };
}, false);
const TryStatement = (node, visitor, context) => {
    const [tryBlockNode, tryBlockVariableState] = TryStatementVisitor(node.tryBlock, visitor, context);
    const catchClause = node.catchClause && TryStatementCatchClauseVisitor(node.catchClause, visitor, context);
    let catchClauseNode;
    if (catchClause) {
        const [{ propertyDeclaration, visitedNode }, catchClauseVariableState] = catchClause;
        const visitedCatchClauseNode = visitedNode;
        catchClauseNode = visitedCatchClauseNode && context.factory.updateCatchClause(visitedCatchClauseNode, visitedCatchClauseNode.variableDeclaration, updateBlock(visitedCatchClauseNode.block, catchClauseVariableState, context, propertyDeclaration));
    }
    const finallyBlock = node.finallyBlock && TryStatementVisitor(node.finallyBlock, visitor, context);
    let finallyBlockNode;
    if (finallyBlock) {
        const [visitedFinallyBlockNode, finallyBlockVariableState] = finallyBlock;
        finallyBlockNode = visitedFinallyBlockNode && updateBlock(visitedFinallyBlockNode, finallyBlockVariableState, context);
    }
    return context.factory.updateTryStatement(node, updateBlock(tryBlockNode, tryBlockVariableState, context), catchClauseNode, finallyBlockNode);
};
exports.TryStatement = TryStatement;
const updateBlock = (node, variableState, context, definedVariablesNames = []) => {
    if (variableState.blockScopeIdentifiers) {
        const declarationNode = (0, variableStatement_1.variableStatement)([
            [variableState.blockScopeIdentifiers, (0, createObject_1.createObject)(definedVariablesNames)]
        ]);
        return context.factory.updateBlock(node, [declarationNode, ...node.statements]);
    }
    return node;
};
