"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsxTransformers = void 0;
const typescript_1 = __importDefault(require("typescript"));
const ArrowFunction_1 = require("./ArrowFunction");
const BinaryExpression_1 = require("./BinaryExpression");
const FunctionDeclaration_1 = require("./FunctionDeclaration");
const FunctionExpression_1 = require("./FunctionExpression");
const Identifier_1 = require("./Identifier");
const jsxToObject_1 = require("./jsxToObject");
const CallExpression_1 = require("./CallExpression");
const createJsxChildrenNode_1 = require("./utils/createJsxChildrenNode");
const PropertyAccessExpressionOrElementAccessExpression_1 = require("./utils/PropertyAccessExpressionOrElementAccessExpression");
const VariableStatement_1 = require("./VariableStatement");
const IfStatement_1 = require("./IfStatement");
const SwitchStatement_1 = require("./SwitchStatement");
const ForStatement_1 = require("./ForStatement");
const ForInStatement_1 = require("./ForInStatement");
const ForOfStatement_1 = require("./ForOfStatement");
const MethodDeclaration_1 = require("./MethodDeclaration");
const ClassStaticBlockDeclaration_1 = require("./ClassStaticBlockDeclaration");
const TryStatement_1 = require("./TryStatement");
const WhileStatement_1 = require("./WhileStatement");
const PostfixUnaryExpression_1 = require("./PostfixUnaryExpression");
const PrefixUnaryExpression_1 = require("./PrefixUnaryExpression");
const SourceFile_1 = require("./SourceFile");
const DoStatement_1 = require("./DoStatement");
exports.jsxTransformers = {
    [typescript_1.default.SyntaxKind.JsxElement]: (node, visitor, context) => {
        const { openingElement: { tagName, attributes }, children } = node;
        return (0, jsxToObject_1.jsxToObject)(visitor, context, tagName, attributes, children);
    },
    [typescript_1.default.SyntaxKind.JsxSelfClosingElement]: (node, visitor, context) => {
        return (0, jsxToObject_1.jsxToObject)(visitor, context, node.tagName, node.attributes, []);
    },
    [typescript_1.default.SyntaxKind.JsxFragment]: (node, visitor, context) => {
        const childrenNode = (0, createJsxChildrenNode_1.createJsxChildrenNode)(visitor, context, node.children);
        return (childrenNode || context.factory.createArrayLiteralExpression([], false));
    },
    [typescript_1.default.SyntaxKind.ArrowFunction]: ArrowFunction_1.ArrowFunction,
    [typescript_1.default.SyntaxKind.FunctionExpression]: FunctionExpression_1.FunctionExpression,
    [typescript_1.default.SyntaxKind.FunctionDeclaration]: FunctionDeclaration_1.FunctionDeclaration,
    [typescript_1.default.SyntaxKind.MethodDeclaration]: MethodDeclaration_1.MethodDeclaration,
    [typescript_1.default.SyntaxKind.ClassStaticBlockDeclaration]: ClassStaticBlockDeclaration_1.ClassStaticBlockDeclaration,
    [typescript_1.default.SyntaxKind.IfStatement]: IfStatement_1.IfStatement,
    [typescript_1.default.SyntaxKind.TryStatement]: TryStatement_1.TryStatement,
    [typescript_1.default.SyntaxKind.SwitchStatement]: SwitchStatement_1.SwitchStatement,
    [typescript_1.default.SyntaxKind.WhileStatement]: WhileStatement_1.WhileStatement,
    [typescript_1.default.SyntaxKind.DoStatement]: DoStatement_1.DoStatement,
    [typescript_1.default.SyntaxKind.ForStatement]: ForStatement_1.ForStatement,
    [typescript_1.default.SyntaxKind.ForInStatement]: ForInStatement_1.ForInStatement,
    [typescript_1.default.SyntaxKind.ForOfStatement]: ForOfStatement_1.ForOfStatement,
    [typescript_1.default.SyntaxKind.PropertyAccessExpression]: PropertyAccessExpressionOrElementAccessExpression_1.PropertyAccessExpressionOrElementAccessExpression,
    [typescript_1.default.SyntaxKind.ElementAccessExpression]: PropertyAccessExpressionOrElementAccessExpression_1.PropertyAccessExpressionOrElementAccessExpression,
    [typescript_1.default.SyntaxKind.CallExpression]: CallExpression_1.CallExpression,
    [typescript_1.default.SyntaxKind.Identifier]: Identifier_1.Identifier,
    [typescript_1.default.SyntaxKind.BinaryExpression]: BinaryExpression_1.BinaryExpression,
    [typescript_1.default.SyntaxKind.VariableStatement]: VariableStatement_1.VariableStatement,
    [typescript_1.default.SyntaxKind.PostfixUnaryExpression]: PostfixUnaryExpression_1.PostfixUnaryExpression,
    [typescript_1.default.SyntaxKind.PrefixUnaryExpression]: PrefixUnaryExpression_1.PrefixUnaryExpression,
    [typescript_1.default.SyntaxKind.SourceFile]: SourceFile_1.SourceFile,
};
