"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsxToObject = void 0;
const typescript_1 = __importDefault(require("typescript"));
const createObject_1 = require("../factoryCode/createObject");
const stringLiteral_1 = require("../factoryCode/stringLiteral");
const createJsxChildrenNode_1 = require("./utils/createJsxChildrenNode");
const forEachJsxAttributes_1 = require("./utils/forEachJsxAttributes");
const useJsxPropRegistration_1 = require("./utils/useJsxPropRegistration");
const createJSXComponent_1 = require("./utils/createJSXComponent");
const arrowFunction_1 = require("../factoryCode/arrowFunction");
const getTagNameString = (tagName, tagNameToString) => {
    if (typescript_1.default.isIdentifier(tagName)) {
        const tagNameString = typescript_1.default.idText(tagName);
        if (/^([a-z]|\d+|\-|\:)*$/.test(tagNameString)) {
            return tagNameString;
        }
    }
    else if (tagName.kind === typescript_1.default.SyntaxKind.ThisKeyword) {
        return "this";
    }
};
const jsxToObject = (visitor, context, tagName, attributes, children) => {
    const childrenNode = (0, createJsxChildrenNode_1.createJsxChildrenNode)(visitor, context, children);
    const tagNameToString = getTagNameString(tagName);
    if (tagNameToString) {
        const objectNodeProperties = [
            [
                tagNameToString,
                childrenNode || context.factory.createArrayLiteralExpression([], false)
            ]
        ];
        const xmlnsNodeProperties = [];
        const eventObjectNodeProperties = [];
        const dynamicObjectNodeProperties = [];
        let haveDefaultXmlns = false;
        (0, forEachJsxAttributes_1.forEachJsxAttributes)(attributes.properties, (attributeName, attributeValueNode) => {
            const attributeNameString = typeof attributeName === "string" ? attributeName : typescript_1.default.idText(attributeName);
            if (/^(on+[A-Z])/.test(attributeNameString)) {
                eventObjectNodeProperties.push([
                    attributeNameString.replace(/^on/, "").toLowerCase(),
                    visitor(attributeValueNode)
                ]);
            }
            else if (typescript_1.default.isJsxText(attributeValueNode) || typescript_1.default.isStringLiteral(attributeValueNode)) {
                const attributeStringNode = (0, stringLiteral_1.stringLiteral)(attributeValueNode.text);
                if (attributeNameString.startsWith("xmlns:")) {
                    xmlnsNodeProperties.push([attributeNameString.replace(/^(xmlns\:)/, ""), attributeStringNode]);
                }
                else if (attributeNameString === "xmlns") {
                    haveDefaultXmlns = true;
                    xmlnsNodeProperties.push(["$D", attributeStringNode]);
                }
                else {
                    objectNodeProperties.push([attributeName, attributeStringNode]);
                }
            }
            else {
                (0, useJsxPropRegistration_1.useJsxPropRegistration)(attributeValueNode, visitor, context, (node, isRegisterNode) => {
                    if (isRegisterNode) {
                        dynamicObjectNodeProperties.push([attributeName, node]);
                    }
                    else {
                        objectNodeProperties.push([attributeName, node]);
                    }
                });
            }
        });
        const elementConstruction = [];
        if (eventObjectNodeProperties.length) {
            elementConstruction.push([
                "E",
                (0, arrowFunction_1.arrowFunction)([], [], (0, createObject_1.createObject)(eventObjectNodeProperties))
            ]);
        }
        if (dynamicObjectNodeProperties.length) {
            elementConstruction.push([
                "D",
                (0, createObject_1.createObject)(dynamicObjectNodeProperties)
            ]);
        }
        let returnElementNode = (0, createObject_1.createObject)(objectNodeProperties);
        if (elementConstruction.length) {
            returnElementNode = (0, createObject_1.createObject)([
                ["$", returnElementNode],
                ...elementConstruction
            ]);
        }
        if (tagNameToString === "svg" && !haveDefaultXmlns) {
            xmlnsNodeProperties.push(['$D', (0, stringLiteral_1.stringLiteral)("http://www.w3.org/2000/svg")]);
        }
        if (xmlnsNodeProperties.length) {
            returnElementNode = (0, createObject_1.createObject)([
                ['$X', returnElementNode],
                ...xmlnsNodeProperties
            ]);
        }
        return returnElementNode;
    }
    return (0, createJSXComponent_1.createJSXComponent)(visitor, context, tagName, attributes, children);
};
exports.jsxToObject = jsxToObject;
