"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBlockVisitor = void 0;
const typescript_1 = __importDefault(require("typescript"));
const getIdentifierState_1 = require("./getIdentifierState");
const createBlockVisitor = (nodeVisitor, isGlobalBlock) => {
    return (node, visitor, context) => {
        const getVariableUniqueIdentifierCache = context.getVariableUniqueIdentifier;
        const variableState = {
            blockScopeIdentifiers: undefined,
            globalScopeIdentifiers: undefined,
        };
        context.getVariableUniqueIdentifier = (flag) => {
            if (isGlobalBlock) {
                return variableState.globalScopeIdentifiers || (variableState.globalScopeIdentifiers = context.factory.createUniqueName("_"));
            }
            if (flag === typescript_1.default.NodeFlags.None) {
                return getVariableUniqueIdentifierCache(flag);
            }
            return variableState.blockScopeIdentifiers || (variableState.blockScopeIdentifiers = context.factory.createUniqueName("_"));
        };
        const visitedNode = (0, getIdentifierState_1.creteManageIdentifierState)(context, isGlobalBlock, () => {
            return nodeVisitor(node, visitor, context);
        });
        context.getVariableUniqueIdentifier = getVariableUniqueIdentifierCache;
        return [visitedNode, variableState];
    };
};
exports.createBlockVisitor = createBlockVisitor;
