"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGlobalBlockNodesVisitor = void 0;
const typescript_1 = __importDefault(require("typescript"));
const createObject_1 = require("../../factoryCode/createObject");
const identifier_1 = require("../../factoryCode/identifier");
const variableStatement_1 = require("../../factoryCode/variableStatement");
const getVariableDeclarationNames_1 = require("../../utils/getVariableDeclarationNames");
const createBlockVisitor_1 = require("./createBlockVisitor");
const createGlobalBlockVisitor = (0, createBlockVisitor_1.createBlockVisitor)((node, visitor, context) => {
    const declarationProperties = [];
    if (!typescript_1.default.isClassStaticBlockDeclaration(node)) {
        for (const parameter of node.parameters) {
            const declarationNamesObject = (0, getVariableDeclarationNames_1.getVariableDeclarationNames)(parameter);
            for (const declarationIdentifierName in declarationNamesObject) {
                context.addIdentifiersChannelCallback(declarationIdentifierName, (identifierState) => {
                    identifierState.declaredFlag = typescript_1.default.NodeFlags.None;
                    identifierState.substituteCallback = (indexIdToUniqueString, declarationIdentifier) => {
                        declarationProperties.push([indexIdToUniqueString, (0, identifier_1.identifier)(declarationIdentifierName)]);
                    };
                });
            }
        }
    }
    return {
        visitedNode: typescript_1.default.visitEachChild(node, visitor, context),
        declarationProperties
    };
}, true);
const createGlobalBlockNodesVisitor = (updateNode) => (node, visitor, context) => {
    const OldRegistrations = context.getJSXPropRegistrationIdentifier;
    context.getJSXPropRegistrationIdentifier = undefined;
    const [{ visitedNode, declarationProperties }, variableState] = createGlobalBlockVisitor(node, visitor, context);
    context.getJSXPropRegistrationIdentifier = OldRegistrations;
    if (variableState.globalScopeIdentifiers) {
        return updateNode(visitedNode, (0, variableStatement_1.variableStatement)([
            [variableState.globalScopeIdentifiers, (0, createObject_1.createObject)(declarationProperties)]
        ]), context);
    }
    return visitedNode;
};
exports.createGlobalBlockNodesVisitor = createGlobalBlockNodesVisitor;
