"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createJSXComponent = void 0;
const typescript_1 = __importDefault(require("typescript"));
const createObject_1 = require("../../factoryCode/createObject");
const createJsxChildrenNode_1 = require("./createJsxChildrenNode");
const forEachJsxAttributes_1 = require("./forEachJsxAttributes");
const useJsxPropRegistration_1 = require("./useJsxPropRegistration");
const stringLiteral_1 = require("../../factoryCode/stringLiteral");
const createJSXComponent = (visitor, context, componentName, attributes, children) => {
    const childrenNode = (0, createJsxChildrenNode_1.createJsxChildrenNode)(visitor, context, children);
    const safeComponentName = typescript_1.default.isJsxNamespacedName(componentName) ? (0, stringLiteral_1.stringLiteral)("componentName") : componentName;
    const propsObjectNodesForFactoryCode = [
        ["$C", safeComponentName],
    ];
    if (childrenNode) {
        propsObjectNodesForFactoryCode.push([
            "i",
            childrenNode,
        ]);
    }
    const dynamicPropsObjectNodesForFactoryCode = [];
    const staticPropsObjectNodesForFactoryCode = [];
    (0, forEachJsxAttributes_1.forEachJsxAttributes)(attributes.properties, (attributeName, attributeValueNode) => {
        (0, useJsxPropRegistration_1.useJsxPropRegistration)(attributeValueNode, visitor, context, (node, isJSXregistererNode) => {
            if (isJSXregistererNode) {
                dynamicPropsObjectNodesForFactoryCode.push([attributeName, node]);
            }
            else {
                staticPropsObjectNodesForFactoryCode.push([attributeName, node]);
            }
        });
    });
    if (staticPropsObjectNodesForFactoryCode.length) {
        propsObjectNodesForFactoryCode.push(["a", (0, createObject_1.createObject)(staticPropsObjectNodesForFactoryCode)]);
    }
    if (dynamicPropsObjectNodesForFactoryCode.length) {
        propsObjectNodesForFactoryCode.push(["d", (0, createObject_1.createObject)(dynamicPropsObjectNodesForFactoryCode)]);
    }
    return (0, createObject_1.createObject)(propsObjectNodesForFactoryCode);
};
exports.createJSXComponent = createJSXComponent;
