"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createJsxChildrenNode = void 0;
const typescript_1 = __importDefault(require("typescript"));
const createObject_1 = require("../../factoryCode/createObject");
const stringLiteral_1 = require("../../factoryCode/stringLiteral");
const safeInitializer_1 = require("./safeInitializer");
const useJsxPropRegistration_1 = require("./useJsxPropRegistration");
const createJsxChildrenNode = (visitor, context, children) => {
    const newChildren = children.reduce((newChildren, child, index) => {
        let currentChild = (0, safeInitializer_1.safeInitializer)(child);
        if (!currentChild) {
            return newChildren;
        }
        if (typescript_1.default.isJsxText(currentChild)) {
            const jsxText = currentChild.text;
            if (jsxText.trim().length === 0) {
                return newChildren;
            }
            newChildren.push((0, stringLiteral_1.stringLiteral)(jsxText));
        }
        else {
            newChildren.push((0, useJsxPropRegistration_1.useJsxPropRegistration)(currentChild, visitor, context, (node, isRegisterNode) => {
                if (isRegisterNode) {
                    return (0, createObject_1.createObject)([
                        ["$D", node]
                    ]);
                }
                return node;
            }));
        }
        return newChildren;
    }, []);
    if (newChildren.length > 1) {
        return context.factory.createArrayLiteralExpression(newChildren, false);
    }
    else if (newChildren.length === 1) {
        return newChildren[0];
    }
};
exports.createJsxChildrenNode = createJsxChildrenNode;
