"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.forEachJsxAttributes = void 0;
const typescript_1 = __importDefault(require("typescript"));
const stringLiteral_1 = require("../../factoryCode/stringLiteral");
const safeInitializer_1 = require("./safeInitializer");
const forEachJsxAttributes = (attributeProperties, forEachCallback) => {
    for (const attribute of attributeProperties) {
        if (attribute.kind === typescript_1.default.SyntaxKind.JsxAttribute) {
            const attributeName = attribute.name;
            let attributeValueNode = (0, safeInitializer_1.safeInitializer)(attribute.initializer);
            if (!attributeValueNode)
                continue;
            if (typescript_1.default.isJsxText(attributeValueNode)) {
                attributeValueNode = (0, stringLiteral_1.stringLiteral)(attributeValueNode.text);
            }
            const attributeNameSafe = typescript_1.default.isJsxNamespacedName(attributeName) ? `${typescript_1.default.idText(attributeName.namespace)}:${typescript_1.default.idText(attributeName.name)}` : attributeName;
            forEachCallback(attributeNameSafe, attributeValueNode);
        }
    }
};
exports.forEachJsxAttributes = forEachJsxAttributes;
