"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.creteManageIdentifierState = void 0;
const typescript_1 = __importDefault(require("typescript"));
const numberToUniqueString_1 = require("../../../utils/numberToUniqueString");
const getIndexId_1 = require("./getIndexId");
const createIdentifiersMap = (context) => {
    const declaredBlockIdentifiers = new Map();
    const addDeclaredIdentifierState = (identifierName, identifierState) => {
        const hasIdentifierState = declaredBlockIdentifiers.has(identifierName);
        if (!hasIdentifierState) {
            let substituteCallback = () => { };
            const { getVariableUniqueIdentifier } = context;
            const indexId = (0, getIndexId_1.getIndexId)();
            let newIdentifierState = {
                isJsx: false,
                isChanged: false,
                declaredFlag: undefined,
                get substituteCallback() {
                    return substituteCallback;
                },
                set substituteCallback(newValue) {
                    const substituteCallbackCache = substituteCallback;
                    substituteCallback = (indexIdToUniqueString, declarationIdentifier) => {
                        newValue(indexIdToUniqueString, declarationIdentifier);
                        substituteCallbackCache(indexIdToUniqueString, declarationIdentifier);
                    };
                    if (this.isJsx && this.isChanged && this.declaredFlag !== undefined) {
                        if (this.declaredFlag !== typescript_1.default.NodeFlags.Const) {
                            substituteCallback((0, numberToUniqueString_1.NumberToUniqueString)(indexId), getVariableUniqueIdentifier(this.declaredFlag));
                        }
                        substituteCallback = () => { };
                    }
                }
            };
            if (identifierState) {
                newIdentifierState.substituteCallback = identifierState.substituteCallback;
            }
            declaredBlockIdentifiers.set(identifierName, newIdentifierState);
        }
        else if (identifierState) {
            const currentIdentifierState = declaredBlockIdentifiers.get(identifierName);
            currentIdentifierState.substituteCallback = identifierState.substituteCallback;
        }
    };
    return { declaredBlockIdentifiers, addDeclaredIdentifierState };
};
const creteManageIdentifierState = (context, isGlobalBlock, visitor) => {
    const { declaredBlockIdentifiers, addDeclaredIdentifierState } = createIdentifiersMap(context);
    let localIdentifiersChannelCallback = () => { };
    const addDeclaredIdentifierParentCache = context.addDeclaredIdentifierState;
    const addIdentifiersChannelCallbackParentCache = context.addIdentifiersChannelCallback;
    const addIdentifiersChannelCallback = (identifierName, addCallback) => {
        const newIdentifiersChannelCallback = (declaredBlockIdentifiers, isGlobalBlock) => {
            const identifierState = declaredBlockIdentifiers.get(identifierName);
            if (identifierState) {
                if (identifierState.declaredFlag === typescript_1.default.NodeFlags.None && !isGlobalBlock) {
                    addIdentifiersChannelCallbackParentCache(identifierName, addCallback);
                    addDeclaredIdentifierParentCache(identifierName, identifierState);
                }
                else {
                    addCallback(identifierState);
                }
            }
            else {
                addIdentifiersChannelCallbackParentCache(identifierName, addCallback);
            }
        };
        const localIdentifiersChannelCallbackCache = localIdentifiersChannelCallback;
        localIdentifiersChannelCallback = (declaredBlockIdentifiers, isGlobalBlock) => {
            localIdentifiersChannelCallbackCache(declaredBlockIdentifiers, isGlobalBlock);
            newIdentifiersChannelCallback(declaredBlockIdentifiers, isGlobalBlock);
        };
    };
    context.addDeclaredIdentifierState = addDeclaredIdentifierState;
    context.addIdentifiersChannelCallback = addIdentifiersChannelCallback;
    const visitedBlock = visitor();
    localIdentifiersChannelCallback(declaredBlockIdentifiers, isGlobalBlock);
    context.addDeclaredIdentifierState = addDeclaredIdentifierParentCache;
    context.addIdentifiersChannelCallback = addIdentifiersChannelCallbackParentCache;
    return visitedBlock;
};
exports.creteManageIdentifierState = creteManageIdentifierState;
